from abc import ABC, abstractmethod
from copy import deepcopy
from selenium import webdriver
from selenium.webdriver.chrome.options import Options


class TestDriver(ABC):

    @abstractmethod
    def open(self):
        pass

    @abstractmethod
    def quit(self):
        pass

    @abstractmethod
    def has_quit(self) -> bool:
        pass


class ChromeTestDriver(TestDriver):

    @classmethod
    @property
    def default_config(cls) -> dict:
        return {
            "headless": False,
            "experimental_flags": None,
            "remote_settings": None
        }

    def __init__(self, config=None):

        if not config:
            config = self.default_config

        self._experimental_flags = None
        self._experimental_flags = False
        self._remote_settings = None
        self._driver = None
        self._session_id = None

        if "remote_settings" in config.keys():
            self._remote_settings = config["remote_settings"]
        if "experimental_flags" in config.keys():
            self._experimental_flags = config["experimental_flags"]
        if "headless" in config.keys():
            self._headless = config["headless"]

    @property
    def experimental_flags(self) -> list:
        ef = self._experimental_flags
        if ef:
            return deepcopy(ef)
        else:
            return []

    @property
    def headless(self) -> bool:
        return self._headless

    @property
    def remote_settings(self) -> dict:
        rs = self._remote_settings
        if rs:
            return deepcopy(rs)
        else:
            return {}

    @property
    def options(self) -> Options:
        """ returns an instance of ChromeOptions based on current configuration """
        options = Options()
        if self.experimental_flags:
            chrome_local_state_prefs = {'browser.enabled_labs_experiments': self.experimental_flags}
            options.add_experimental_option('localState', chrome_local_state_prefs)
        if self.headless:
            options.add_argument("--headless")
        return options

    def open(self):
        if self.remote_settings:
            kwargs = self.remote_settings
            kwargs["options"] = self.options
            self._driver = webdriver.Remote(**kwargs)
        else:
            self._driver = webdriver.Chrome(options=self.options)
        self._session_id = self._driver.session_id
        self._driver.maximize_window()

    def quit(self):
        self._driver.quit()
        self._session_id = None

    def has_quit(self):
        if self._session_id:
            return False
        return True

    def get_native_driver(self) -> webdriver.Chrome:
        return self._driver



""" Unittest for the abstract_command.py module """

# python packages
import unittest
import time

# vision packages
from nemonet.engines.graph import Action
from nemonet.seleniumwebdriver.abstract_command import AbstractCommand


""" - - - classes to be used only for testing purposes - - - """

class _DummySubclass(AbstractCommand):

    """ a valid subclass of AbstractCommand """

    def execute_action(self, action: Action, driver):
         return action.getElementType() + " " + driver


""" - - - unittests - - - """

class TestAbstractCommand(unittest.TestCase):

    def setUp(self):

        self.dummy_graph_action = Action("some_xpathstring", "TEST_TYPE", value="value", wait_after= 3, key = "something")
        self.dummy_driver = "TEST_DRIVER"

    def tearDown(self):

        pass

    def test_abstract_command_should_not_instantiate(self):

        with self.assertRaises(TypeError, msg="Abstract Class should not instantiate"):
            AbstractCommand()

    def test_subclasses_should_instantiate(self):

        _DummySubclass()

    def test_execute_command_sleep(self):

        sleep = 5
        before = time.time()
        _DummySubclass().execute(self.dummy_graph_action, self.dummy_driver, sleep_time=sleep)
        after = time.time()

        self.assertTrue(after - before > ((sleep * 2) - 1), "Vision command did not sleep long enough")

    def test_execute_command_screenshot(self):

        # TODO
        # _DummySubclass().execute(self.dummy_graph_action, self.dummy_driver, filename="Hello, World")
        pass

    def test_execute_action(self):

        actual      = _DummySubclass().execute_action(self.dummy_graph_action, self.dummy_driver)
        expected    = self.dummy_graph_action.getElementType() + " " + self.dummy_driver

        self.assertEqual(expected, actual)

    def test_log(self):

        def extract_words_from_line(string : str, words : int):
            temp = string.split(" ")
            return ' '.join(temp[-words:]).replace('\n','')

        # what I want to log
        expect = "Hello, world!"

        # what is actually logged
        _DummySubclass().log(expect)
        with open('vision.log', 'r') as log:
            for line in log:
                pass
            last_line = line
        actual = extract_words_from_line(last_line, 2)

        self.assertEqual(expect, actual, msg=f"logging not working as expected")


if __name__ == '__main__':
    unittest.main()





import random
import string

from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select

from nemonet.cvision.computer_vision import ComputerVision
from nemonet.engines.cache_store import KeyValueStore
from nemonet.engines.graph import Action
from nemonet.seleniumwebdriver.abstract_command import AbstractCommand
from nemonet.seleniumwebdriver.page_capture import PageCapturing


class ClickableCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        xpath_str = action.getXpathStr()
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_element_located((By.XPATH, xpath_str)))
        actions = ActionChains(driver)
        actions.move_to_element(element).perform()
        actions.click()
        actions.perform()


class ClickableDoubleCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        xpath_str = action.getXpathStr()
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_element_located((By.XPATH, xpath_str)))
        actions = ActionChains(driver)
        actions.move_to_element(element).perform()
        actions.double_click(element)
        actions.perform()


class TextableCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        rStr = action.getValue()
        if (rStr == None):
            rStr = ''.join([random.choice(string.ascii_letters + string.digits) for n in range(32)])
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_element_located((By.XPATH, action.getXpathStr())))
        actions = ActionChains(driver)
        actions.move_to_element(element).perform()
        actions.click()
        actions.send_keys(rStr)
        actions.perform()


class JSexecCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        driver.execute_script(action.getXpathStr())


class SelectableCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_all_elements_located((By.XPATH, action.getXpathStr())))
        actions = ActionChains(driver)
        actions.move_to_element(element[2]).perform()
        actions.click()
        actions.perform()


class OpenURLCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        driver.get(str(action.getValue()))


class ScreenshotCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        filename = action.getValue()
        capture = PageCapturing(driver)
        capture.capture_save(file_name_cpatured=filename)


class ClearTextableCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        rStr = action.getValue()
        if rStr == None:
            rStr = ''.join([random.choice(string.ascii_letters + string.digits) for n in range(32)])
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_element_located((By.XPATH, action.getXpathStr())))
        driver.execute_script("arguments[0].scrollIntoView();", element)
        driver.execute_script("arguments[0].value = '';", "")
        self.sleep(0.25)
        actions = ActionChains(driver)
        element.clear()
        actions.move_to_element(element).perform()
        element.clear()
        actions.click()
        actions.send_keys(rStr)
        actions.perform()


class ClearTextableTabCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        rStr = action.getValue()
        if rStr == None:
            rStr = ''.join([random.choice(string.ascii_letters + string.digits) for n in range(32)])
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_element_located((By.XPATH, action.getXpathStr())))
        actions = ActionChains(driver)
        element.clear()
        actions.move_to_element(element).perform()
        actions.click()
        actions.send_keys(rStr)
        actions.send_keys(Keys.TAB)
        actions.perform()


class TextableEnterCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        rStr = action.getValue()
        if rStr == None:
            rStr = ''.join([random.choice(string.ascii_letters + string.digits) for n in range(32)])
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_element_located((By.XPATH, action.getXpathStr())))
        actions = ActionChains(driver)
        actions.move_to_element(element)
        actions.click()
        actions.send_keys(rStr)
        actions.send_keys(Keys.RETURN)
        actions.perform()


class ComparePNGCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        arguments = eval(action.getValue())
        filenameA = arguments[0]
        filenameB = arguments[1]
        cv = ComputerVision()
        result = cv.diff(filenameA, filenameB)
        assert result, 'Difference between %s and %s' % (filenameA, filenameB)


class ComparePNGHashCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        arguments = eval(action.getValue())
        filenameA = arguments[0]
        filenameB = arguments[1]
        cv = ComputerVision()
        result = cv.diff_with_phash_image_hash(filenameA, filenameB)
        assert result, 'Difference between %s and %s' % (filenameA, filenameB)


class WaitCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        self.sleep(int(action.getValue()))


class ScrollTopCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        driver.execute_script("window.scrollTo(0,0);")


class ScrollBottomCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight)")


class BrowserTabsGoToCurrentCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        driver.switch_to.window(driver.current_window_handle)


class TextCurrentPositionCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        actions = ActionChains(driver)
        actions.send_keys(action.getValue())
        actions.perform()


class TextCurrentPositionStampCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        text = action.getValue()
        actions = ActionChains(driver)
        text = text + " " + ''.join([random.choice(string.ascii_letters + string.digits) for n in range(32)])
        actions.send_keys(text)
        actions.perform()


class TabCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        actions = ActionChains(driver)
        actions.send_keys(Keys.TAB)
        actions.perform()


class ClickableRightCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_element_located((By.XPATH, action.getXpathStr())))
        actions = ActionChains(driver)
        actions.move_to_element(element).perform()
        actions.context_click(element)
        actions.perform()


class DragAndDropCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        wait = WebDriverWait(driver, 60)
        xpath_from = action.getXpathStr()
        xpath_to = action.getValue()
        element_from = wait.until(ec.visibility_of_element_located((By.XPATH, xpath_from)))
        element_to = wait.until(ec.visibility_of_element_located((By.XPATH, xpath_to)))
        actions = ActionChains(driver)
        actions.move_to_element(element_from).perform()
        actions.drag_and_drop(element_from, element_to)
        actions.perform()


class DragAndDropMouseCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        wait = WebDriverWait(driver, 60)
        xpath_from = action.getXpathStr()
        to_xpath = action.getValue()
        element_from = wait.until(ec.visibility_of_element_located((By.XPATH, xpath_from)))
        element_to = wait.until(ec.visibility_of_element_located((By.XPATH, to_xpath)))
        actions = ActionChains(driver)
        actions.click(element_from)
        actions.perform()
        self.sleep(0.25)
        actions.click_and_hold(element_from)
        actions.move_to_element(element_to)
        actions.perform()
        self.sleep(0.25)
        actions.release()
        self.sleep(0.25)
        actions.perform()


class GoToFrameCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        frame_nbr = action.getValue()
        driver.switch_to.frame(int(frame_nbr))


class DragAndDropWithOffsetCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        wait = WebDriverWait(driver, 60)
        xpath_from = action.getXpathStr()
        offsets = action.getValue()
        offsets = offsets.split(',')
        element_from = wait.until(ec.visibility_of_element_located((By.XPATH, xpath_from)))
        actions = ActionChains(driver)
        actions.move_to_element(element_from).perform()
        self.sleep(0.25)
        actions.drag_and_drop_by_offset(element_from, offsets[0], offsets[1])
        self.sleep(0.25)
        actions.perform()


class SwitchToAlertAndConfirmCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        the_alert = driver.switch_to.alert
        the_alert.accept()


class RemoveHTMLElementCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        wait = WebDriverWait(driver, 60)
        css_selector = action.getValue()
        element_css_selector = wait.until(ec.presence_of_element_located((By.CSS_SELECTOR, css_selector)))
        self.sleep(0.25)
        driver.execute_script("arguments[0].scrollIntoView();", element_css_selector)
        self.sleep(0.25)
        driver.execute_script("arguments[0].parentNode.removeChild(arguments[0]);", element_css_selector)


class SaveCurrentURLCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        kv_store = KeyValueStore()
        kv_store.add(action.get_key(), driver.current_url)
        kv_store.close()


class SplitStoredURLCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        url_key = action.get_key()
        kv_store = KeyValueStore()
        url = kv_store.get_value_as_str(url_key)
        split_url = url.split('//')[1].split('/')
        counter = 1
        for section in split_url:
            if len(section) > 0:
                url_key_nbr = url_key + "_" + str(counter)
                counter += 1
                kv_store.add(url_key_nbr, section)
        kv_store.close()


class FormatStringsCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        value = action.getValue()
        key = action.get_key()
        kv_store = KeyValueStore()
        reformated_str = kv_store.replace_formated_key(value)
        kv_store.add(key, reformated_str)
        kv_store.close()


class StoreStringCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        value = action.getValue()
        key = action.get_key()
        kv_store = KeyValueStore()
        kv_store.add(key, value)
        kv_store.close()


class SplitURLAndStoreCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        value = action.getValue()
        key_prefix = action.get_key()
        split_url = value.split('//')[1].split('/')
        kv_store = KeyValueStore()
        counter = 1
        for section in split_url:
            if len(section) > 0:
                url_key_nbr = key_prefix + "_" + str(counter)
                counter += 1
                kv_store.add(url_key_nbr, section)
        kv_store.close()


class SelectFromDropDownCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        wait = WebDriverWait(driver, 60)
        locator = (By.XPATH, action.getXpathStr())
        element = wait.until(ec.element_to_be_clickable(locator), message='Cannot locate {}'.format(locator))
        driver.execute_script("arguments[0].scrollIntoView(true);", element)
        Select(element).select_by_visible_text(action.getValue())


class SiteSettingsCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        site_setting_url_and_tab_nbr_list = action.getValue().split('|')
        url = site_setting_url_and_tab_nbr_list[0]
        tab_nbr = site_setting_url_and_tab_nbr_list[1]
        set_back_url = driver.current_url
        driver.get(str(url))  # open url
        actions = ActionChains(driver)
        actions.send_keys(Keys.TAB * int(tab_nbr))
        actions.send_keys(Keys.ARROW_DOWN)
        actions.perform()
        self.sleep(0.25)
        driver.get(str(set_back_url))  # open url


class TabsAndEnterCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        tab_nbr = action.getValue()
        actions = ActionChains(driver)
        actions.send_keys(Keys.TAB * int(tab_nbr))
        actions.send_keys(Keys.RETURN)
        actions.perform()
        self.sleep(0.25)


class TabsAndTextCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        values = action.getValue().split(':')
        tab_nbr = values[0]
        actions = ActionChains(driver)
        actions.send_keys(Keys.TAB * int(tab_nbr))
        actions.send_keys(values[1])
        actions.perform()
        self.sleep(0.25)


class BrowserTabsAddCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        nbr_of_tabs = int(action.getValue())
        self.log(driver.current_window_handle)
        for nbr in range(nbr_of_tabs):
            tab_name = "TAB_" + str(nbr)
            driver.execute_script("window.open('about:blank', '%s');" % (tab_name))
            self.sleep(0.5)
            driver.switch_to.window(tab_name)
            self.log(driver.current_window_handle)


class BrowserTabsGoToCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        tab_nbr = int(action.getValue())
        handles = driver.window_handles
        self.log(handles)
        driver.switch_to.window(handles[tab_nbr])


class LogCurrentURLCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        self.log(driver.current_url)


class OpenStoredURLCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        kv_store = KeyValueStore()
        stored_key = action.get_key()
        url = kv_store.get_value_as_str(stored_key)
        self.log(url)
        driver.get(str(url))  # open url


class SetEnvironmentVariable(AbstractCommand):
    def execute_action(self, action: Action, driver):
        value = action.getValue()
        key_to_store = action.get_key()
        value_to_store = action.getValue()
        kv_store = KeyValueStore()
        kv_store.add(key_to_store, value_to_store)
        kv_store.close()


class SelectorExists(AbstractCommand):
    """
        Check if the specified selector is available in the screen
        On failure let it throw an exception
    """

    def execute_action(self, action: Action, driver):
        xpath_str = action.getXpathStr()
        wait = WebDriverWait(driver, 60)
        wait.until(ec.visibility_of_element_located((By.XPATH, xpath_str)))


class GetValueAndCompare(AbstractCommand):
    """
        Get the value from the selector
        On failure let it throw an exception
    """

    def execute_action(self, action: Action, driver):
        xpath_str = action.getXpathStr()
        value = str(action.getValue())
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_element_located((By.XPATH, xpath_str)))
        assert value == str(element.text)


class GetTextAndStore(AbstractCommand):
    """
        Get the value from the selector and store the value
        On failure let it throw an exception
    """

    def execute_action(self, action: Action, driver):
        xpath_str = action.getXpathStr()
        key = str(action.get_key())
        wait = WebDriverWait(driver, 60)
        element = wait.until(ec.visibility_of_element_located((By.XPATH, xpath_str)))
        kv_store = KeyValueStore()
        kv_store.add(key, str(element.text))
        kv_store.close()



""" Unittest for the abstract_command.py module """

# python packages
import unittest

# vision packages
from nemonet.engines.graph import Action
from nemonet.seleniumwebdriver.commands import Command


class TestCommands(unittest.TestCase):

    def setUp(self):
        self.dummy_graph_action = Action("some_xpathstring", "DUMMY", value="value", wait_after=3, key="something")
        self.dummy_driver = "TEST_DRIVER"
        self.dummy_command = Command(self.dummy_driver)

    def tearDown(self):
        pass

    def test_dummy_command(self):
        dummy_command_class = self.dummy_command.execute_command(self.dummy_graph_action)
        self.assertIsNone(dummy_command_class, msg="dummy command class should be None")

    def test_unknown_command(self):
        unknown_graph_action = Action("this type should not exist", "@m$acL9QiXaSKdRe")
        with self.assertRaises(AssertionError, msg="Unknown commands should throw an error"):
            self.dummy_command.execute_command(unknown_graph_action)


if __name__ == '__main__':
    unittest.main()



# Created by Jan Rummens at 8/01/2021
import configparser

config = configparser.ConfigParser()
# is expected to have the cfg in the current folder
config.read('vision.cfg')  # failure wrong name => KeyError
import sys
root_path = config['PLUGIN']['RootDir']
sys.path.append(root_path)
import logging.config

logging.config.fileConfig(fname='vision_logger.conf', disable_existing_loggers=False)
logger = logging.getLogger(__name__)

from nemonet.runner.runner import Runner
import typer
import traceback

app = typer.Typer()


@app.command()
def scenario(name: str, useconfig: bool = False):
    try:
        if useconfig:
            runner = Runner.from_json_file(runner_config="runner_config.json")
        else:
            runner = Runner()
        runner.execute_scenario(name)
    except ValueError:
        typer.echo(f"invalid commandline")
        logger.debug("Fatal Error ValueError", exc_info=True)
    except FileNotFoundError as e:
        typer.echo(e)
    except Exception as err:
        traceback.print_tb(err.__traceback__)
        logger.debug("Fatal Error Exception", exc_info=True)
    finally:
        pass



import unittest
from nemonet.runner.webdriver import ChromeTestDriver

class RunnerDriverTestCase(unittest.TestCase):

    def test_chrometestdriver_config_init(self):

        """ tests if ChromeTestDrivers can be correctly instantiated
            with a specific configuration """

        flag = ["dummy"]
        settings = {"dummy": "remote_settings"}
        config_full = {
            "headless" : True,
            "experimental_flags" : flag,
            "remote_settings" : settings
        }
        config_empty = {
            "headless": False,
            "experimental_flags": None,
            "remote_settings": None
        }

        c_full = ChromeTestDriver(config=config_full)

        msg = "ChromeTestDriver init with configuration issue for :"
        self.assertTrue(c_full.headless, msg=msg + "headless")
        self.assertEqual(c_full.experimental_flags, flag, msg=msg + "experimental flags")
        self.assertEqual(c_full.remote_settings, settings, msg=msg + "remote settings")

        c_empty = ChromeTestDriver(config=config_empty)
        self.assertFalse(c_empty.headless, msg=msg + "headless")
        self.assertEqual(c_empty.experimental_flags, [], msg=msg + "experimental flags")
        self.assertEqual(c_empty.remote_settings, {}, msg=msg + "experimental flags")


from nemonet.runner.webdriver import TestDriver, ChromeTestDriver

class WebdriverError(Exception):
    pass

class SeleniumTool:

    @classmethod
    @property
    def default_config(cls) -> dict:
        return {
            "webdriver": "chrome",
            "webdriver_config": ChromeTestDriver.default_config
        }

    @classmethod
    def setup_driver(cls, config=None) -> TestDriver:

        """ returns an instance of a TestDriver based on given selenium config """

        if not config:
            config = cls.default_config

        webdriver = config["webdriver"]

        if webdriver == "chrome":
            return ChromeTestDriver(config["webdriver_config"])
        else:
            raise(WebdriverError(f"{webdriver} is not a supported webdriver"))


# Created by Jan Rummens at 12/01/2021
import unittest
import copy
import threading

from nemonet.runner.runner import Runner

import logging.config

logging.config.fileConfig(fname='../../vision_logger.conf', disable_existing_loggers=False)
logger = logging.getLogger(__name__)


class RemoteTestCase(unittest.TestCase):

    """ Before running these tests, make sure Selenium Grid has been set up locally """

    def setUp(self) -> None:
        self.config = {
            "driver": {
                "tool": "selenium",
                "tool_config": {
                    "webdriver": "chrome",
                    "webdriver_config": {
                        "headless": True,
                        "experimental_flags": [
                            "same-site-by-default-cookies@2",
                            "cookies-without-same-site-must-be-secure@2"],
                        "remote_settings": None
                    }
                }
            },
            "screen_recording": {
                "enable": False
            }
        }

    def test_remote_chrome(self):
        # Before running this test, make sure Selenium Grid has been set up locally
        remote_config = copy.deepcopy(self.config)
        remote_settings = {"command_executor": "http://127.0.0.1:4444"}
        remote_config["driver"]["tool_config"]["webdriver_config"]["remote_settings"] = remote_settings
        remote_runner = Runner(config=remote_config)

        remote_runner.execute_scenario("command-current-pos")

    def test_remote_chrome_in_parallel(self):
        # Before running this test, make sure Selenium Grid has been set up locally
        # MAKE SURE TO VISUALLY CONFIRM (without headless) THIS TEST. Errors from multi-threaded processes
        # are not captured by the unittest framework
        threads = []
        number_of_sessions = range(3)
        for session in number_of_sessions:
            t = threading.Thread(target=self.test_remote_chrome)
            threads.append(t)
            t.start()

        for t in threads:
            t.join()


if __name__ == '__main__':
    unittest.main()



from nemonet.seleniumwebdriver.abstract_command import AbstractCommand
from nemonet.engines.graph import Action
import inspect

class ReadAndStore(AbstractCommand):
    def execute_action(self, action: Action, driver):
        m = inspect.getmodule(self)
        with open('ReadAndStore.txt', 'w') as f:
            f.write(m.__name__)
            f.close()

    def command_name(self) -> str:
        return 'READ_AND_STORE'



# Created by Jan Rummens at 5/01/2021
import os
import unittest
import json
from nemonet.runner.runner import Runner, RunnerError


class RunnerConfigTestCase(unittest.TestCase):

    def setUp(self) -> None:
        self.filename = 'runner_config.json'
        self.default_config_dict = Runner.default_config

        with open( self.filename, 'w') as fp:
            json.dump(self.default_config_dict, fp, indent=4)

    def tearDown(self) -> None:
        os.remove(self.filename)

    def test_runner_init_from_file(self):
        """ There should be no difference between instantiating a Runner "normally" and
            from a json file """

        r1 = Runner(self.default_config_dict)
        r2 = Runner.from_json_file(self.filename)

        for data in [r1.config, r2.config]:
            self.assertTrue( list( data.keys() ) == ['driver', 'screen_recording'] )
            self.assertTrue( list( data["driver"].keys() ) == ['tool', 'tool_config'] )
            self.assertTrue( list( data["screen_recording"].keys()) == ['enable'] )

    def test_runner_raises_error(self):

        config = self.default_config_dict
        config["driver"]["tool"] = "unsupported tool"

        with self.assertRaises(RunnerError):
            Runner(config=config)


if __name__ == '__main__':
    unittest.main()



# Created by Jan Rummens at 12/01/2021
import unittest
import copy
from nemonet.runner.runner import Runner
from nemonet.engines.cache_store import KeyValueStore

import logging.config

logging.config.fileConfig(fname='../../vision_logger.conf', disable_existing_loggers=False)
logger = logging.getLogger(__name__)


class CommandsTestCase(unittest.TestCase):

    def setUp(self) -> None:
        self.config = {
            "driver": {
                "tool": "selenium",
                "tool_config": {
                    "webdriver": "chrome",
                    "webdriver_config": {
                        "headless": True,
                        "experimental_flags": [
                            "same-site-by-default-cookies@2",
                            "cookies-without-same-site-must-be-secure@2"],
                        "remote_settings": None
                    }
                }
            },
            "screen_recording": {
                "enable": False
            }
        }

        self.runner = Runner(config=self.config)
        kv_store = KeyValueStore()
        kv_store.add("VISION_ENV_SCREENSHOTS","false")
        kv_store.close()

    def tearDown(self) -> None:
        # quit driver of runner that weren't used (didn't call execute_scenario)
        if not self.runner.driver.has_quit():
            self.runner.driver.quit()

    def test_plugins(self):
        self.runner.execute_scenario("commands-plugins")
        with open('ReadAndStore.txt', encoding="utf-8") as f:
            read_data = f.read()
            self.assertTrue(read_data == 'nemonet.tests.units.plugin.vision_my_custom_1')
            f.close()


if __name__ == '__main__':
    unittest.main()



# Created by Jan Rummens at 3/12/2020
import logging
logger = logging.getLogger(__name__)

from nemonet.engines.graph import Graph
from nemonet.engines.sequencer import Sequences
from nemonet.seleniumwebdriver.commands import Command
from nemonet.screencast.recording import ScreenRecorder
from nemonet.runner.tool import SeleniumTool
import json
import time


class RunnerError(Exception):
    pass


class Runner(object):

    @classmethod
    @property
    def default_config(cls):
        return {
            "driver": {
                "tool": "selenium",
                "tool_config": SeleniumTool.default_config
            },
            "screen_recording": {
                "enable": False
            }
        }

    def __init__(self, config=None):

        self.config = config
        self.driver = None
        self.has_passed = False

        if not self.config:
            self.config = self.default_config

        self._set_up_driver()

    def _set_up_driver(self):
        tool = self.config["driver"]["tool"]
        tool_config = self.config["driver"]["tool_config"]

        if tool == "selenium":
            self.driver = SeleniumTool.setup_driver(tool_config)
            self.driver.open()
        else:
            raise(RunnerError(f"{tool} is not a supported testing tool"))

    def turn_on_recording(self):
        if not self.is_recording:
            self.screenrecording.start()
            self.is_recording = True
            time.sleep(0.25)

    def turn_off_recording(self):
        if self.is_recording:
            self.screenrecording.stop()
            time.sleep(0.25)

    def scenario_passed(self):
        return self.has_passed

    def execute_scenario(self, xml_files_name):
        if self.config["screen_recording"]['enable']:
            self.is_recording = False
            self.screenrecording = ScreenRecorder()
            self.turn_on_recording()
        graph = Graph()
        graph.build(xml_files_name)
        sequences = Sequences(graph)
        commands = Command(self.driver.get_native_driver())
        commands.executeSequences(sequences, graph)
        self.driver.quit()
        self.has_passed = True

    @classmethod
    def from_json_file(cls, runner_config_filepath):
        with open(runner_config_filepath, 'r') as fp:
            return cls(config=json.load(fp))



# Created by Jan Rummens at 12/01/2021
import unittest
import copy
from nemonet.runner.runner import Runner
from nemonet.engines.cache_store import KeyValueStore

import logging.config

logging.config.fileConfig(fname='../../vision_logger.conf', disable_existing_loggers=False)
logger = logging.getLogger(__name__)


class CommandsTestCase(unittest.TestCase):

    def setUp(self) -> None:
        self.config = {
            "driver": {
                "tool": "selenium",
                "tool_config": {
                    "webdriver": "chrome",
                    "webdriver_config": {
                        "headless": True,
                        "experimental_flags": [
                            "same-site-by-default-cookies@2",
                            "cookies-without-same-site-must-be-secure@2"],
                        "remote_settings": None
                    }
                }
            },
            "screen_recording": {
                "enable": False
            }
        }

        self.runner = Runner(config=self.config)
        kv_store = KeyValueStore()
        kv_store.add("VISION_ENV_SCREENSHOTS","false")
        kv_store.close()

    def tearDown(self) -> None:
        # quit driver of runner that weren't used (didn't call execute_scenario)
        if not self.runner.driver.has_quit():
            self.runner.driver.quit()

    def test_chrome_options(self):
        self.runner.execute_scenario("dummy")

    def test_chrome_headless_current_pos(self):
        headless_config = copy.deepcopy(self.config)
        headless_config["driver"]["tool_config"]["webdriver_config"]["headless"] = True
        headless_runner = Runner(config=headless_config)

        headless_runner.execute_scenario("command-current-pos")

    def test_save_records(self):
        self.runner.execute_scenario("commands-save-records")

    def test_double_click(self):
        self.runner.execute_scenario("commands-double-mouse-click")

    def test_drag_and_drop(self):
        self.runner.execute_scenario("commands-drag-and-drop")

    def test_drag_and_drop_mouse(self):
        self.runner.execute_scenario("commands-drag-and-drop-mouse")

    def test_drag_and_drop_offset(self):
        self.runner.execute_scenario("commands-drag-and-drop-offset")

    def test_explicit_waits(self):
        self.runner.execute_scenario("commands-explicit-waits")

    def test_remove_html_element(self):
        self.runner.execute_scenario("commands-remove-html-element")

    def test_right_mouse(self):
        self.runner.execute_scenario("command-right-mouse")

    def test_tabs(self):
        self.runner.execute_scenario("command-tabs")

    def test_current_pos(self):
        self.runner.execute_scenario("command-current-pos")

    def test_site_settings(self):
        # chrome only
        self.runner.execute_scenario("commands-site-settings")

    def test_open_formatted_url(self):
        self.runner.execute_scenario("commands-reformat-url")

    def test_tab_and_text(self):
        self.runner.execute_scenario("command-tab-and-text")

    def test_set_env(self):
        self.runner.execute_scenario("command-set-env")

    def test_get_val(self):
        self.runner.execute_scenario("command-get_value")

    def test_sel_exists(self):
        self.runner.execute_scenario("command-sel_exists")

    def test_get_val_and_store(self):
        self.runner.execute_scenario("command-get_value_and_store")

    def test_plugins(self):
        self.runner.execute_scenario("commands-plugins")


if __name__ == '__main__':
    unittest.main()



'''
Created on 13 feb. 2018

@author: ex03210
'''
import configparser
import inspect
import logging
import sys
from importlib import import_module
from os import walk

logger = logging.getLogger(__name__)

from nemonet.seleniumwebdriver.command_list import *


class Command(object):
    """
        All kind of selenium actions and more
    """

    def __init__(self, driver=None):
        """Constructor"""
        self.driver = driver
        self._wait = WebDriverWait(self.driver, 60)
        # on construction, we need to provide the plugin commands
        try:
            self.build_plugins()
        except Exception as e:
            logger.info("No plugins added " + str(e))

    def executeSequences(self, seq, g):

        for step in g.getSetup():
            self.execute_command(step.getAction())

        pa = seq.get()
        for l in pa:
            # logger.info("Execute sequence=%s" % (str(l)))
            for el in l:
                a = g.getVertex(el).getAction()
                self.execute_command(a)

    def execute_command(self, action: Action):
        """
        TODO : error handling
        :param action:
        :return:
        """
        command_type = action.getElementType()

        try:
            command_class = commands[command_type]
            if (command_class == None):
                pass  # Dummy Command for testing purposes
            else:
                c = command_class()
                c.execute(action, self.driver)
                return c

        except KeyError:
            # TODO : AssertionError is too generic, in need of an UnknownCommandException
            logger.debug("Fatal Error KeyError ", exc_info=True)
            assert False, f"{command_type} is not a known command"

    def build_plugins(self):
        # use a configuration file approach
        config = configparser.ConfigParser()
        # is expected to have the cfg in the current folder
        config.read('vision.cfg')  # failure wrong name => KeyError
        logger.debug("Config file is read")
        # review this way of resolving the path to the plugin directory
        # config file needed to specify the path to the plugin folder
        mypath = config['PLUGIN']['PluginDir']
        logger.debug("Plugin Folder= %s" % (mypath))
        vision_modules = []
        for (dirpath, dirnames, filenames) in walk(mypath):
            vision_modules.extend(filenames)
        filtered = list(filter(lambda vision_module: vision_module.startswith("vision_"), vision_modules))
        # remove compiled occurrences .pyc
        filtered = [item for item in filtered if '.pyc' not in item]
        filtered = [w.removesuffix('.py') for w in filtered]
        # look for a plugin directory and determine the plugin package directory
        # example below retrieves instance of the module
        # maybe best is to add to config file
        classes_as_spec = []  # result classes after checking the class protocol
        for mod in filtered:
            logger.debug("Import module= %s" % (config['PLUGIN']['PackageName']))
            m2 = import_module('.' + mod, package=config['PLUGIN']['PackageName'])
            logger.debug("Import module instance= %s" % (m2))
            # get the classes from the module
            classes = [cls_name for cls_name, cls_obj in inspect.getmembers(sys.modules[m2.__name__]) if
                       inspect.isclass(cls_obj)]
            logger.debug("classes from the module= %s" % (classes))
            # inspect what class has the correct method signature
            for cls in classes:
                is_vision_class = getattr(m2, cls)
                if hasattr(is_vision_class,
                           "execute_action") and is_vision_class.__base__.__name__ == 'AbstractCommand':
                    classes_as_spec.append(is_vision_class)
                    #below gives failure
                    logger.info("Added plugin " + is_vision_class().command_name())
                    #commands[is_vision_class(self).command_name()] = is_vision_class
                    commands[is_vision_class().command_name()] = is_vision_class

# TODO : this variable is exposed and needs a better location
commands = {
    'DUMMY': None,
    'CLICKABLE': ClickableCommand,
    'CLICKABLE_DOUBLE': ClickableDoubleCommand,
    'TEXTABLE': TextableCommand,
    'JSexec': JSexecCommand,
    'SELECTABLE': SelectableCommand,
    'OPENURL': OpenURLCommand,
    'SCREENSHOT': ScreenshotCommand,
    'CLEARTEXTABLE': ClearTextableCommand,
    'CLEARTEXTABLETAB': ClearTextableTabCommand,
    'TEXTABLE-ENTER': TextableEnterCommand,
    'COMPAREPNG': ComparePNGCommand,
    'COMPAREPNGPHASH': ComparePNGHashCommand,
    'WAIT': WaitCommand,
    'SCROLLTOP': ScrollTopCommand,
    'SCROLLBOTTOM': ScrollBottomCommand,
    'TAB': TabCommand,
    'CLICKABLE_RIGHT': ClickableRightCommand,
    'DRAG_AND_DROP': DragAndDropCommand,
    'GOTO_FRAME': GoToFrameCommand,
    'SAVE_CURRENT_URL': SaveCurrentURLCommand,
    'SPLIT_STORED_URL': SplitStoredURLCommand,
    'FORMAT_STRINGS': FormatStringsCommand,
    'STORE_STRING': StoreStringCommand,
    'SITE_SETTINGS': SiteSettingsCommand,
    'TABS_AND_ENTER': TabsAndEnterCommand,
    'TABS_AND_TEXT': TabsAndTextCommand,
    'BROWSER_TABS_ADD': BrowserTabsAddCommand,
    'BROWSER_TABS_GOTO': BrowserTabsGoToCommand,
    'LOG_CURRENT_URL': LogCurrentURLCommand,
    'OPEN_STORED_URL': OpenStoredURLCommand,
    'BROWSER_TABS_GOTO_CURRENT': BrowserTabsGoToCurrentCommand,
    'TEXT_CURRENT_POSITION': TextCurrentPositionCommand,
    'TEXT_CURRENT_POSITION_STAMP': TextCurrentPositionStampCommand,
    'DRAG_AND_DROP_MOUSE': DragAndDropMouseCommand,
    'DRAG_AND_DROP_WITH_OFFSET': DragAndDropWithOffsetCommand,
    'SWITCH_TO_ALERT_AND_CONFIRM': SwitchToAlertAndConfirmCommand,
    'REMOVE_HTML_ELEMENT': RemoveHTMLElementCommand,
    'SPLIT_URL_AND_STORE': SplitURLAndStoreCommand,
    'SELECT_FROM_DROP_DOWN': SelectFromDropDownCommand,
    'SET_VISION_ENV': SetEnvironmentVariable,
    'SEL_EXISTS': SelectorExists,
    'GET_VALUE_AND_COMPARE': GetValueAndCompare,
    'GET_TEXT_AND_STORE': GetTextAndStore
}



import unittest
from abc import ABC, abstractmethod

""" - - - classes for demonstration purposes - - -"""


class _Demo_AbstractClass(ABC):
    """
        an ABC for demonstration purposes with abstract properties,
        abstract methods, and more
    """

    def __init__(self, foo=1):
        self.foo = foo

    @abstractmethod
    def time_wait(self):
        return "Abstract"

    @abstractmethod
    def execute_action(self):
        return "Abstract"

    @abstractmethod
    def screenshot(self):
        return "Abstract"

    def execute_command(self):
        res = ""
        res += self.time_wait()
        res += self.screenshot()
        res += self.execute_action()
        res += self.time_wait()
        res += self.screenshot()
        res += self.time_wait()
        return res


class _Demo_CompletelyInvalid_Subclass(_Demo_AbstractClass):
    """ a completely invalid subclass for demonstration purposes """

    pass  # implements none of the required abstract methods


class _Demo_Invalid_Subclass(_Demo_AbstractClass):
    """ an invalid but almost ok subclass for demonstration purposes """

    def time_wait(self):
        pass

    def execute_action(self):
        pass

    # not all abstract methods are implemented


class _Demo_Valid_NotConcreteSubclass(_Demo_AbstractClass):
    """ a valid subclass that doesn't do anything concrete """

    def time_wait(self):
        return super(_Demo_Valid_NotConcreteSubclass, self).time_wait()

    def execute_action(self):
        return super(_Demo_Valid_NotConcreteSubclass, self).execute_action()

    def screenshot(self):
        return super(_Demo_Valid_NotConcreteSubclass, self).screenshot()


class _Demo_Valid_ConcreteSubclass(_Demo_AbstractClass):
    """ a valid subclass that doesn't do anything concrete """

    def time_wait(self):
        return 'Concrete'

    def execute_action(self):
        return 'Concrete'

    def screenshot(self):
        return 'Concrete'


class _Demo_Valid_SemiConcreteCommand(_Demo_AbstractClass):
    """ a valid subclass that does some things concrete """

    def time_wait(self):
        return 'Concrete'

    def execute_action(self):
        return super(_Demo_Valid_SemiConcreteCommand, self).execute_action()

    def screenshot(self):
        return 'Concrete'


class _Demo_Valid_HyperConcreteSubclass(_Demo_AbstractClass):
    """ a valid subclass that doesn't do anything concrete """

    def time_wait(self):
        return 'Concrete'

    def execute_action(self):
        return 'Concrete'

    def screenshot(self):
        return 'Concrete'

    def execute_command(self):
        return "I have overwritten my superclass's implementation of non-abstract methods"


class _Demo_INIT(_Demo_AbstractClass):

    def __init__(self, foo):
        super().__init__(foo)

    def time_wait(self):
        return 'Concrete'

    def execute_action(self):
        return 'Concrete'

    def screenshot(self):
        return 'Concrete'


class _DemoPropertyAbstract(ABC):
    def __init__(self, name):
        self._name = name

    @property
    @abstractmethod
    def name(self):
        return self._name


class _DemoProperties(_DemoPropertyAbstract):

    @property
    def name(self):
        return self._name

""" - - - Demonstration tests - - - """


class TestDemoABC(unittest.TestCase):
    """ Unittest to illustrate functionality of python's abc.ABC class """

    def test_demo_abc_do_not_instantiate(self):
        with self.assertRaises(TypeError):
            _Demo_AbstractClass()

    def test_demo_abstract_methods_have_to_be_implemented(self):
        with self.assertRaises(TypeError, msg="ABC subclasses should implement abstract methods"):
            _Demo_CompletelyInvalid_Subclass()

        with self.assertRaises(TypeError, msg="ABC subclasses should implement all abstract methods"):
            _Demo_Invalid_Subclass()

        _Demo_Valid_ConcreteSubclass()

    def test_demo_concreteness_of_subclasses(self):
        expect = "ConcreteConcreteConcreteConcreteConcreteConcrete"
        actual = _Demo_Valid_ConcreteSubclass().execute_command()

        self.assertEqual(expect, actual, msg="ABC subclasses can have their own implementations")

        expect = "ConcreteConcreteAbstractConcreteConcreteConcrete"
        actual = _Demo_Valid_SemiConcreteCommand().execute_command()

        self.assertEqual(expect, actual, msg="ABC subclasses can use the abstract implementations")

        expect = "AbstractAbstractAbstractAbstractAbstractAbstract"
        actual = _Demo_Valid_NotConcreteSubclass().execute_command()

        self.assertEqual(expect, actual, msg="ABC subclasses can use all the abstract implementations")

        expect = "I have overwritten my superclass's implementation of non-abstract methods"
        actual = _Demo_Valid_HyperConcreteSubclass().execute_command()

        self.assertEqual(expect, actual, msg="ABC subclasses can overwrite non-abstract implementations")

    def test_demo_abstract_possibilities(self):
        # init
        foo = 2
        demo = _Demo_INIT(foo)
        self.assertEqual(demo.foo, 2)

    def testNameProperty(self):
        #property
        demo = _DemoProperties('Jan')
        self.assertEqual(demo.name, 'Jan')


if __name__ == '__main__':
    unittest.main()



import logging
logger = logging.getLogger(__name__)

import time
from abc import ABC, abstractmethod
from datetime import datetime
import uuid

from nemonet.engines.graph import Action
from nemonet.seleniumwebdriver.page_capture import PageCapturing
from nemonet.engines.cache_store import KeyValueStore

class AbstractCommand(ABC):

    @abstractmethod
    def execute_action(self, action: Action, driver):
        pass  # does nothing by default

    def execute(self, action: Action, driver, sleep_time=0.25, filename=None):
        """ Executes given action as a vision command """

        self.__sleep(sleep_time)
        self.__screenshot(action, driver)
        self.__log(str(action))
        self.execute_action(action, driver)
        self.__sleep(sleep_time)
        self.__screenshot(action, driver)
        self.__sleep(action.get_wait_after())

    def log(self, string):
        self.__log(string)

    def sleep(self, second):
        self.__sleep(second)

    def __screenshot(self, action, driver):
        """ Takes a screenshot of the current state of [ TERMINOLOGY ] using given driver
            and saves it as a .png file with given filename [ WHERE? ].
            Fails silently when given filename is None. """

        try:
            kv_store = KeyValueStore()
            value_true_false = kv_store.get_value_as_str("VISION_ENV_SCREENSHOTS")
            kv_store.close()

            if value_true_false.lower() == 'false':
                return
        except:
            logger.debug("Error KeyValue Store ", exc_info=True)

        if action == None:
            return

        if action.getElementType() == 'SCREENSHOT':
            return

        capture = PageCapturing(driver)
        now = datetime.now()
        filename = "screenshot_" + now.strftime("%Y%m%d-%H%M%S.%f") + "-" \
                   + action.getElementType() + "-" \
                   + str(uuid.uuid4()) + ".png"
        capture.capture_save(file_name_cpatured=filename)

    def __sleep(self, seconds):
        time.sleep(seconds)

    def __log(self, string):
        # TODO : recomputation should be avoided
        # set up
        # log
        logger.debug(string)



from nemonet.seleniumwebdriver.abstract_command import AbstractCommand
from nemonet.engines.graph import Action


class ScreenDump(AbstractCommand):
    def execute_action(self, action: Action, driver):
        pass

    def command_name(self) -> str:
        return 'SCREEN_DUMP'


class FancyCommand(AbstractCommand):
    def execute_action(self, action: Action, driver):
        pass

    def command_name(self) -> str:
        return 'FANCY_COMMAND'



import unittest
from nemonet.runner.tool import SeleniumTool, WebdriverError

class RunnerToolTestCase(unittest.TestCase):

    def test_setup_driver(self):

        """ only tests if the method runs for all webdrivers """

        SeleniumTool.setup_driver(None)

        chrome_config = {
            "webdriver": "chrome",
            "webdriver_config": None
        }
        SeleniumTool.setup_driver(chrome_config)


    def test_setup_driver_raises_error(self):

        unsupported_config = {
            "webdriver": "unknown webdriver",
            "webdriver_config": None
        }
        with self.assertRaises(WebdriverError):
            SeleniumTool.setup_driver(unsupported_config)
